@@ -15,7 +15,8 @@ more advanced memory and task tools of dask to be used)
 DASK_SCATTER_TIMEOUT = 120
 
 
-def amusr_regress_dask(X, Y, priors, prior_weight, n_tasks, genes, tfs, G, remove_autoregulation=True):
+def amusr_regress_dask(X, Y, priors, prior_weight, n_tasks, genes, tfs, G, remove_autoregulation=True,
+                       lambda_Bs=None, lambda_Ss=None, Cs=None, Ss=None, regression_function=None):
     """
     Execute multitask (AMUSR)
 
@@ -28,6 +29,9 @@ def amusr_regress_dask(X, Y, priors, prior_weight, n_tasks, genes, tfs, G, remov
     from inferelator.regression.amusr_regression import format_prior, run_regression_EBIC
     DaskController = MPControl.client
 
+    # Allows injecting a regression function for testing
+    regression_function = run_regression_EBIC if regression_function is None else regression_function
+
     # Gets genes, n_tasks, prior_weight, and remove_autoregulation from regress_dask()
     # Other arguments are passed in
     def regression_maker(j, x_df, y_list, prior, tf):
@@ -49,7 +53,8 @@ def amusr_regress_dask(X, Y, priors, prior_weight, n_tasks, genes, tfs, G, remov
             tasks.append(k)  # [T,]
 
         prior = format_prior(prior, gene, tasks, prior_weight)
-        return j, run_regression_EBIC(x, y, tf, tasks, gene, prior)
+        return j, regression_function(x, y, tf, tasks, gene, prior,
+                                      lambda_Bs=lambda_Bs, lambda_Ss=lambda_Ss, Cs=Cs, Ss=Ss)
 
     def response_maker(y_df, i):
         y = []
@@ -252,6 +257,28 @@ def build_mi_array_dask(X, Y, bins, logtype):
     return mi
 
 
+def dask_map(func, *args, **kwargs):
+    """
+    Dask map
+
+    :param func: function to map
+    :type func: callable
+    :param args: positional arguments for func
+    :type args: iterable
+    :param kwargs: keyword (non-iterable) arguments for func. Keywords will be passed to dask client.map.
+    :return: List of mapped results
+    :rtype: list
+    """
+
+    assert MPControl.is_dask()
+
+    def _func_caller(f, i, *a, **k):
+        return i, f(*a, **k)
+
+    return process_futures_into_list([MPControl.client.client.submit(_func_caller, func, i, *za, **kwargs)
+                                      for i, za in enumerate(zip(*args))])
+
+
 def process_futures_into_list(future_list, raise_on_error=True, check_results=True):
     """
     Take a list of futures and turn them into a list of results
